<?php declare(strict_types=1);


namespace Pinpoint\DropShipping\Model;

use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\Api\ExtensibleDataObjectConverter;
use Magento\Framework\Api\ExtensionAttribute\JoinProcessorInterface;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Reflection\DataObjectProcessor;
use Magento\Store\Model\StoreManagerInterface;
use Pinpoint\DropShipping\Api\Data\DropShipItemInterfaceFactory;
use Pinpoint\DropShipping\Api\Data\DropShipItemSearchResultsInterfaceFactory;
use Pinpoint\DropShipping\Api\DropShipItemRepositoryInterface;
use Pinpoint\DropShipping\Model\ResourceModel\DropShipItem as ResourceDropShipItem;
use Pinpoint\DropShipping\Model\ResourceModel\DropShipItem\CollectionFactory as DropShipItemCollectionFactory;


class DropShipItemRepository implements DropShipItemRepositoryInterface
{

    protected $resource;

    protected $dropShipItemFactory;

    protected $dropShipItemCollectionFactory;

    protected $searchResultsFactory;

    protected $dataObjectHelper;

    protected $dataObjectProcessor;

    protected $dataDropShipItemFactory;

    protected $extensionAttributesJoinProcessor;

    private $storeManager;

    private $collectionProcessor;

    protected $extensibleDataObjectConverter;

    /**
     * @param ResourceDropShipItem $resource
     * @param DropShipItemFactory $dropShipItemFactory
     * @param DropShipItemInterfaceFactory $dataDropShipItemFactory
     * @param DropShipItemCollectionFactory $dropShipItemCollectionFactory
     * @param DropShipItemSearchResultsInterfaceFactory $searchResultsFactory
     * @param DataObjectHelper $dataObjectHelper
     * @param DataObjectProcessor $dataObjectProcessor
     * @param StoreManagerInterface $storeManager
     * @param CollectionProcessorInterface $collectionProcessor
     * @param JoinProcessorInterface $extensionAttributesJoinProcessor
     * @param ExtensibleDataObjectConverter $extensibleDataObjectConverter
     */
    public function __construct(
        ResourceDropShipItem $resource,
        DropShipItemFactory $dropShipItemFactory,
        DropShipItemInterfaceFactory $dataDropShipItemFactory,
        DropShipItemCollectionFactory $dropShipItemCollectionFactory,
        DropShipItemSearchResultsInterfaceFactory $searchResultsFactory,
        DataObjectHelper $dataObjectHelper,
        DataObjectProcessor $dataObjectProcessor,
        StoreManagerInterface $storeManager,
        CollectionProcessorInterface $collectionProcessor,
        JoinProcessorInterface $extensionAttributesJoinProcessor,
        ExtensibleDataObjectConverter $extensibleDataObjectConverter
    ) {
        $this->resource = $resource;
        $this->dropShipItemFactory = $dropShipItemFactory;
        $this->dropShipItemCollectionFactory = $dropShipItemCollectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        $this->dataDropShipItemFactory = $dataDropShipItemFactory;
        $this->dataObjectProcessor = $dataObjectProcessor;
        $this->storeManager = $storeManager;
        $this->collectionProcessor = $collectionProcessor;
        $this->extensionAttributesJoinProcessor = $extensionAttributesJoinProcessor;
        $this->extensibleDataObjectConverter = $extensibleDataObjectConverter;
    }

    /**
     * {@inheritdoc}
     */
    public function save(
        \Pinpoint\DropShipping\Api\Data\DropShipItemInterface $dropShipItem
    ) {
        /* if (empty($dropShipItem->getStoreId())) {
            $storeId = $this->storeManager->getStore()->getId();
            $dropShipItem->setStoreId($storeId);
        } */

        $dropShipItemData = $this->extensibleDataObjectConverter->toNestedArray(
            $dropShipItem,
            [],
            \Pinpoint\DropShipping\Api\Data\DropShipItemInterface::class
        );

        $dropShipItemModel = $this->dropShipItemFactory->create()->setData($dropShipItemData);

        try {
            $this->resource->save($dropShipItemModel);
        } catch (\Exception $exception) {
            throw new CouldNotSaveException(__(
                'Could not save the dropShipItem: %1',
                $exception->getMessage()
            ));
        }
        return $dropShipItemModel->getDataModel();
    }

    /**
     * {@inheritdoc}
     */
    public function get($dropShipItemId)
    {
        $dropShipItem = $this->dropShipItemFactory->create();
        $this->resource->load($dropShipItem, $dropShipItemId);
        if (!$dropShipItem->getId()) {
            throw new NoSuchEntityException(__('DropShipItem with id "%1" does not exist.', $dropShipItemId));
        }
        return $dropShipItem->getDataModel();
    }

    public function getById($dropShipItemId)
    {
        $dropShipItem = $this->dropShipItemFactory->create();
        $this->resource->load($dropShipItem, $dropShipItemId);
        if (!$dropShipItem->getId()) {
            throw new NoSuchEntityException(__('DropShipItem with id "%1" does not exist.', $dropShipItemId));
        }
        return $dropShipItem;
    }

    /**
     * {@inheritdoc}
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $criteria
    ) {
        $collection = $this->dropShipItemCollectionFactory->create();

        $this->extensionAttributesJoinProcessor->process(
            $collection,
            \Pinpoint\DropShipping\Api\Data\DropShipItemInterface::class
        );

        $this->collectionProcessor->process($criteria, $collection);

        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);

        $items = [];
        foreach ($collection as $model) {
            $items[] = $model->getDataModel();
        }

        $searchResults->setItems($items);
        $searchResults->setTotalCount($collection->getSize());
        return $searchResults;
    }

    /**
     * {@inheritdoc}
     */
    public function delete(
        \Pinpoint\DropShipping\Api\Data\DropShipItemInterface $dropShipItem
    ) {
        try {
            $dropShipItemModel = $this->dropShipItemFactory->create();
            $this->resource->load($dropShipItemModel, $dropShipItem->getDropshipitemId());
            $this->resource->delete($dropShipItemModel);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__(
                'Could not delete the DropShipItem: %1',
                $exception->getMessage()
            ));
        }
        return true;
    }

    /**
     * {@inheritdoc}
     */
    public function deleteById($dropShipItemId)
    {
        return $this->delete($this->get($dropShipItemId));
    }

    public function getDropshipDate($supplierId, $orderId, $method)
    {
        $connection = $this->resource->getConnection();

        $select = $connection->select()
            ->from(
                ['pdd' =>   $this->resource->getConnection()->getTableName('pinpoint_dropshipping_dropshipitem')],
                'MIN(created_at)'
            )->where(
                'pdd.method=?',
                $method
            )->where(
                'pdd.order_id=?',
                $orderId
            )->where(
                'pdd.supplier_id=?',
                $supplierId
            );

        return $connection->fetchOne($select);
    }
}

